// functions defined in the respective .cpp files
void expose_matrices();
void expose_vectors();
void expose_boxes();
void expose_quaternion();
void expose_complex(); // does nothing if _COMPLEX_SUPPORT is not #defined
void expose_converters();
